@@ -14,7 +14,10 @@
 
 from __future__ import absolute_import, division, print_function
 
-from nacl._lib import lib
+from nacl import exceptions as exc
+from nacl._sodium import ffi, lib
+
+randombytes_SEEDBYTES = lib.randombytes_seedbytes()
 
 
 def randombytes(size):
@@ -25,6 +28,24 @@ def randombytes(size):
     :param size: int
     :rtype: bytes
     """
-    buf = lib.ffi.new("unsigned char[]", size)
+    buf = ffi.new("unsigned char[]", size)
     lib.randombytes(buf, size)
-    return lib.ffi.buffer(buf, size)[:]
+    return ffi.buffer(buf, size)[:]
+
+
+def randombytes_buf_deterministic(size, seed):
+    """
+    Returns ``size`` number of deterministically generated pseudorandom bytes
+    from a seed
+
+    :param size: int
+    :param seed: bytes
+    :rtype: bytes
+    """
+    if len(seed) != randombytes_SEEDBYTES:
+        raise exc.TypeError("Deterministic random bytes must be generated "
+                            "from 32 bytes")
+
+    buf = ffi.new("unsigned char[]", size)
+    lib.randombytes_buf_deterministic(buf, size, seed)
+    return ffi.buffer(buf, size)[:]
