@@ -20,7 +20,7 @@ import os
 import sys
 import unicodedata as ud
 
-from hypothesis import given, settings, unlimited
+from hypothesis import given, settings
 from hypothesis.strategies import integers, text
 
 import pytest
@@ -84,6 +84,8 @@ def argon2id_raw_ref():
     return vectors
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(
     ("size", "password", "salt", "opslimit", "memlimit", "expected"),
     [
@@ -105,6 +107,8 @@ def test_kdf_scryptsalsa208sha256(size, password, salt,
     assert res == expected
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(("password", ), [
     (
         b"The quick brown fox jumps over the lazy dog.",
@@ -116,6 +120,8 @@ def test_scryptsalsa208sha256_random(password):
     assert h1 != h2
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(("password", ), [
     (
         b"The quick brown fox jumps over the lazy dog.",
@@ -128,6 +134,8 @@ def test_scryptsalsa208sha256_verify(password):
     )
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(("password", ), [
     (
         b"The quick brown fox jumps over the lazy dog.",
@@ -141,6 +149,8 @@ def test_scryptsalsa208sha256_verify_incorrect(password):
         )
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(
     ("size", "password", "salt", "opslimit", "memlimit"),
     [
@@ -160,6 +170,8 @@ def test_wrong_salt_length(size, password, salt,
                                              opslimit, memlimit)
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(
     ("passwd_hash", "password"),
     [
@@ -175,6 +187,8 @@ def test_wrong_hash_length(passwd_hash, password):
                                                 password)
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(
     ("size", "password", "salt", "opslimit", "memlimit"),
     [
@@ -194,6 +208,8 @@ def test_kdf_wrong_salt_length(size, password, salt,
                                              opslimit, memlimit)
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(
     ("passwd_hash", "password"),
     [
@@ -209,6 +225,8 @@ def test_str_verify_wrong_hash_length(passwd_hash, password):
                                                 password)
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(
     ("size", "password", "salt", "opslimit", "memlimit", "expected"),
     [
@@ -230,6 +248,8 @@ def test_scrypt_kdf(size, password, salt,
     assert res == expected
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(("password", ), [
     (
         b"The quick brown fox jumps over the lazy dog.",
@@ -241,6 +261,8 @@ def test_scrypt_random(password):
     assert h1 != h2
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(("password", ), [
     (
         b"The quick brown fox jumps over the lazy dog.",
@@ -253,6 +275,8 @@ def test_scrypt_verify(password):
     )
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(("password", ), [
     (
         b"The quick brown fox jumps over the lazy dog.",
@@ -266,6 +290,8 @@ def test_scrypt_verify_incorrect(password):
         )
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(
     ("size", "password", "salt", "opslimit", "memlimit"),
     [
@@ -285,6 +311,8 @@ def test_wrong_scrypt_salt_length(size, password, salt,
                                opslimit, memlimit)
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(
     ("passwd_hash", "password"),
     [
@@ -300,6 +328,8 @@ def test_wrong_scrypt_hash_length(passwd_hash, password):
                                   password)
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(
     ("size", "password", "salt", "opslimit", "memlimit"),
     [
@@ -367,6 +397,8 @@ def test_variable_limits(opslimit, memlimit, n, r, p):
     assert rp == p
 
 
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
 @pytest.mark.parametrize(
     ("passwd_hash", "password"),
     [
@@ -410,7 +442,7 @@ def test_str_verify_argon2_ref_fail(password_hash, password):
        integers(min_value=1024 * 1024,
                 max_value=16 * 1024 * 1024)
        )
-@settings(deadline=None, max_examples=20, timeout=unlimited)
+@settings(deadline=None, max_examples=20)
 def test_argon2i_str_and_verify(password, ops, mem):
     _psw = password.encode('utf-8')
     pw_hash = nacl.pwhash.argon2i.str(_psw, opslimit=ops, memlimit=mem)
@@ -424,7 +456,7 @@ def test_argon2i_str_and_verify(password, ops, mem):
        integers(min_value=1024 * 1024,
                 max_value=16 * 1024 * 1024)
        )
-@settings(deadline=None, max_examples=20, timeout=unlimited)
+@settings(deadline=None, max_examples=20)
 def test_argon2id_str_and_verify(password, ops, mem):
     _psw = password.encode('utf-8')
     pw_hash = nacl.pwhash.argon2id.str(_psw, opslimit=ops, memlimit=mem)
@@ -438,7 +470,7 @@ def test_argon2id_str_and_verify(password, ops, mem):
        integers(min_value=1024 * 1024,
                 max_value=16 * 1024 * 1024)
        )
-@settings(deadline=None, max_examples=20, timeout=unlimited)
+@settings(deadline=None, max_examples=20)
 def test_argon2i_str_and_verify_fail(password, ops, mem):
     _psw = password.encode('utf-8')
     pw_hash = nacl.pwhash.argon2i.str(_psw, opslimit=ops, memlimit=mem)
@@ -447,7 +479,7 @@ def test_argon2i_str_and_verify_fail(password, ops, mem):
 
 
 @given(text(alphabet=PASSWD_CHARS, min_size=5, max_size=20))
-@settings(deadline=None, max_examples=5, timeout=unlimited)
+@settings(deadline=None, max_examples=5)
 def test_pwhash_str_and_verify(password):
     _psw = password.encode('utf-8')
 
@@ -467,6 +499,14 @@ def test_pwhash_str_and_verify(password):
     a2id_res = nacl.pwhash.verify(a2id_hash, _psw)
     assert a2id_res is True
 
+
+@pytest.mark.skipif(not nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires full build of libsodium")
+@given(text(alphabet=PASSWD_CHARS, min_size=5, max_size=20))
+@settings(deadline=None, max_examples=5)
+def test_pwhash_scrypt_str_and_verify(password):
+    _psw = password.encode('utf-8')
+
     scrypt_hash = nacl.pwhash.scrypt.str(
         _psw,
         opslimit=nacl.pwhash.scrypt.OPSLIMIT_INTERACTIVE,
@@ -483,6 +523,13 @@ def test_invalid_modular_scrypt_prefix():
         nacl.pwhash.verify(invalid_modular_hash, psw)
 
 
+def test_crypt_prefix_error():
+    psw = b'always invalid password'
+    invalid_modular_hash = b'$invalid_prefix$'
+    with pytest.raises(exc.CryptPrefixError):
+        nacl.pwhash.verify(invalid_modular_hash, psw)
+
+
 @pytest.mark.parametrize(("dk_size", "password", "salt",
                           "iters", "mem_kb", "pwhash"),
                          argon2i_raw_ref())
@@ -552,3 +599,29 @@ def test_check_limits_for_unknown_algorithm():
     from nacl.bindings.crypto_pwhash import _check_argon2_limits_alg
     with pytest.raises(exc.TypeError):
         _check_argon2_limits_alg(4, 1024, -1)
+
+
+@pytest.mark.skipif(nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires minimal build of libsodium")
+def test_scryptsalsa208sha256_unavailable():
+    empty = b""
+
+    with pytest.raises(exc.UnavailableError):
+        nacl.pwhash.kdf_scryptsalsa208sha256(0, empty, empty)
+    with pytest.raises(exc.UnavailableError):
+        nacl.pwhash.scryptsalsa208sha256_str(empty)
+    with pytest.raises(exc.UnavailableError):
+        nacl.pwhash.verify_scryptsalsa208sha256(empty, empty)
+
+
+@pytest.mark.skipif(nacl.pwhash.scrypt.AVAILABLE,
+                    reason="Requires minimal build of libsodium")
+def test_scrypt_unavailable():
+    empty = b""
+
+    with pytest.raises(exc.UnavailableError):
+        nacl.pwhash.scrypt.kdf(0, empty, empty)
+    with pytest.raises(exc.UnavailableError):
+        nacl.pwhash.scrypt.str(empty)
+    with pytest.raises(exc.UnavailableError):
+        nacl.pwhash.scrypt.verify(empty, empty)
